@@ -168,7 +168,7 @@ class SiteTree(object):
         sitetree = self.get_cache_entry('sitetrees', alias)
         if not sitetree:
             sitetree = TreeItem.objects.select_related('parent', 'tree').\
-                   filter(tree__alias__exact=alias).order_by('parent', 'sort_order')
+                   filter(tree__alias__exact=alias).order_by('parent__sort_order', 'sort_order')
             self.set_cache_entry('sitetrees', alias, sitetree)
             sitetree_needs_caching = True
 
@@ -394,7 +394,7 @@ class SiteTree(object):
                 branch_id = current_item.parent.id
                 parent_ids.append(branch_id)
             elif branch_id.isdigit():
-                parent_ids.append(branch_id)
+                parent_ids.append(int(branch_id))
             else:
                 parent_aliases.append(branch_id)
 
